@@ -20,8 +20,14 @@
 #include <X11/extensions/Xrender.h>
 #include <X11/Xft/Xft.h>
 
+#if RENDER_MAJOR > 0 || RENDER_MINOR >= 5
+#define HAVE_XRENDER
+#endif
+
 #include "gpeclockface.h"
 
+#define BACKGROUND_IMAGE TRUE
+
 #ifdef BACKGROUND_IMAGE
 static GdkPixbuf *clock_background, *clock_background_24;
 static guint background_width, background_height;
@@ -45,11 +51,14 @@ struct _GpeClockFace
 
   guint x_offset, y_offset;
 
-  GtkAdjustment *hour_adj, *minute_adj;
+  GtkAdjustment *hour_adj, *minute_adj, *second_adj;
 
+#ifdef HAVE_XRENDER
   gboolean can_render;
   XftDraw *draw;
   Picture image_pict, src_pict;
+#endif
+
   GdkPixmap *backing_pixmap;
   GdkGC *backing_gc;
   
@@ -57,9 +66,9 @@ struct _GpeClockFace
   gboolean do_grabs;
   gboolean grabbed;
 
-  gulong hour_handler, minute_handler;
+  gulong hour_handler, minute_handler, second_handler;
 
-  double hour_angle, minute_angle;
+  double hour_angle, minute_angle, second_angle;
 };
 
 struct _GpeClockFaceClass
@@ -67,29 +76,41 @@ struct _GpeClockFaceClass
   GtkWidgetClass parent_class;
 };
 
+#ifndef HAVE_XRENDER
+typedef struct
+{
+  double x, y;
+} FakeXPointDouble;
+#endif
+
 static void
 draw_hand (GpeClockFace *clock,
 	   double angle, 
-	   guint length)
+	   guint length,
+	   guint thick)
 {
   GdkPoint points[5];
+#ifdef HAVE_XRENDER
   XPointDouble poly[5];
+#else
+  FakeXPointDouble poly[5];
+#endif
   int i;
   double sa = sin (angle), ca = cos (angle);
 
-  points[0].x = 3;
+  points[0].x = thick;
   points[0].y = 0;
 
-  points[1].x = 3;
+  points[1].x = thick;
   points[1].y = length - 4;
 
   points[2].x = 0;
   points[2].y = length;
 
-  points[3].x = -3;
+  points[3].x = -thick;
   points[3].y = length - 4;
 
-  points[4].x = -3;
+  points[4].x = -thick;
   points[4].y = 0;
 
   for (i = 0; i < 5; i++)
@@ -102,6 +123,7 @@ draw_hand (GpeClockFace *clock,
       poly[i].y = -y + clock->y_offset + clock->radius;
     }
 
+#ifdef HAVE_XRENDER
   if (clock->can_render)
     {
       XRenderCompositeDoublePoly (GDK_WINDOW_XDISPLAY (clock->widget.window),
@@ -114,8 +136,9 @@ draw_hand (GpeClockFace *clock,
 				  EvenOddRule);
     }
   else
+#endif
     {
-      for (i = 0; i < 5; i++)
+      for (i = 0; i < thick; i++)
 	{
 	  points[i].x = poly[i].x;
 	  points[i].y = poly[i].y;
@@ -132,6 +155,8 @@ draw_hand (GpeClockFace *clock,
 static void
 hand_angles (GpeClockFace *clock)
 {
+  if (clock->second_adj != NULL)
+    clock->second_angle = gtk_adjustment_get_value (clock->second_adj) * 2 * M_PI / 60;
   clock->minute_angle = gtk_adjustment_get_value (clock->minute_adj) * 2 * M_PI / 60;
   clock->hour_angle = gtk_adjustment_get_value (clock->hour_adj) * 2 * M_PI / 12
     + gtk_adjustment_get_value (clock->minute_adj) * 2 * M_PI / 60 / 12;
@@ -269,8 +294,10 @@ gpe_clock_face_expose (GtkWidget *widget,
 		+ gtk_adjustment_get_value (clock->minute_adj) * -(360 * 64 / 24) / 60,
 		180 * 64);
 
-  draw_hand (clock, clock->minute_angle, 7 * clock->radius / 8);
-  draw_hand (clock, clock->hour_angle, 3 * clock->radius / 5);
+  if (clock->second_adj != NULL)
+    draw_hand (clock, clock->second_angle, 7 * clock->radius / 8, 1);
+  draw_hand (clock, clock->minute_angle, 7 * clock->radius / 8, 3);
+  draw_hand (clock, clock->hour_angle, 3 * clock->radius / 5, 3);
 
   gdk_draw_drawable (drawable, gc, clock->backing_pixmap, 0, 0, 0, 0, 
 		     widget->allocation.width, widget->allocation.height);
@@ -301,15 +328,24 @@ gpe_clock_face_button_press (GtkWidget *w, GdkEventButton *b)
   gint y_start = b->y - clock->y_offset - clock->radius;
   double r = sqrt (x_start * x_start + y_start * y_start);
   double start_angle = angle_from_xy (x_start, y_start);
-  double hour_angle = gtk_adjustment_get_value (clock->hour_adj) * 2 * M_PI / 12;
+  double hour_angle = ((int)gtk_adjustment_get_value (clock->hour_adj) % 12) * 2 * M_PI / 12;
   double minute_angle = gtk_adjustment_get_value (clock->minute_adj) * 2 * M_PI / 60;
+  double second_angle;
+
+  if (clock->second_adj != NULL)
+    second_angle = gtk_adjustment_get_value (clock->second_adj) * 2 * M_PI / 60;
+  else
+    second_angle = 0; /* just to avoid gcc warning */
   
   hour_angle -= start_angle;
   minute_angle -= start_angle;
   if (hour_angle < 0)
     hour_angle = -hour_angle;
-  if (minute_angle < 0)
-    minute_angle = -minute_angle;
+  if (clock->second_adj != NULL) {
+    second_angle -= start_angle;
+    if (second_angle < 0)
+      second_angle = -second_angle;
+  }
 
   if (r > (clock->radius * 3 / 5) || (minute_angle < hour_angle))
     clock->dragging_minute_hand = TRUE;
@@ -409,8 +445,12 @@ gpe_clock_face_prepare_xrender (GtkWidget *widget)
 					  widget->allocation.height,
 					  gdk_drawable_get_depth (widget->window));
 
+  gdk_draw_rectangle (clock->backing_pixmap, widget->style->bg_gc[GTK_STATE_NORMAL],
+		      TRUE, 0, 0, widget->allocation.width, widget->allocation.height);
+
   clock->backing_gc = gdk_gc_new (clock->backing_pixmap);
 
+#ifdef HAVE_XRENDER
   if (clock->can_render)
     {
       XRenderPictureAttributes att;
@@ -432,6 +472,7 @@ gpe_clock_face_prepare_xrender (GtkWidget *widget)
       att.poly_edge = PolyEdgeSmooth;
       XRenderChangePicture (dpy, clock->image_pict, CPPolyEdge, &att);
     }
+#endif
 }
 
 static void
@@ -439,6 +480,7 @@ gpe_clock_face_unprepare_xrender (GtkWidget *widget)
 {
   GpeClockFace *clock = GPE_CLOCK_FACE (widget);
 
+#ifdef HAVE_XRENDER
   if (clock->can_render)
     {
       XftDrawDestroy (clock->draw);
@@ -447,6 +489,7 @@ gpe_clock_face_unprepare_xrender (GtkWidget *widget)
       clock->src_pict = 0;
       clock->draw = NULL;
     }
+#endif
       
   if (clock->backing_gc)
     {
@@ -484,7 +527,9 @@ gpe_clock_face_realize (GtkWidget *widget)
   GpeClockFace *clock = GPE_CLOCK_FACE (widget);
   GdkWindowAttr attributes;
   gint attributes_mask;
+#ifdef HAVE_XRENDER
   int major, minor;
+#endif
 
   GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
     
@@ -509,12 +554,14 @@ gpe_clock_face_realize (GtkWidget *widget)
   widget->style = gtk_style_attach (widget->style, widget->window);
   gtk_style_set_background (widget->style, widget->window, GTK_STATE_ACTIVE);
 
+#ifdef HAVE_XRENDER
   if (XRenderQueryVersion (GDK_WINDOW_XDISPLAY (widget->window), &major, &minor) &&
       (major > 0 ||
        (major == 0 && minor >= 4)))
     clock->can_render = TRUE;
   else
     clock->can_render = FALSE;
+#endif
 
   gpe_clock_face_prepare_xrender (widget);
 
@@ -522,6 +569,9 @@ gpe_clock_face_realize (GtkWidget *widget)
 					  G_CALLBACK (adjustment_value_changed), clock);
   clock->minute_handler = g_signal_connect (G_OBJECT (clock->minute_adj), "value_changed", 
 					    G_CALLBACK (adjustment_value_changed), clock);
+  if (clock->second_adj != NULL)
+    clock->second_handler = g_signal_connect (G_OBJECT (clock->second_adj), "value_changed", 
+					      G_CALLBACK (adjustment_value_changed), clock);
 }
 
 GdkBitmap *
@@ -569,6 +619,8 @@ gpe_clock_face_unrealize (GtkWidget *widget)
 
   g_signal_handler_disconnect (clock->hour_adj, clock->hour_handler);
   g_signal_handler_disconnect (clock->minute_adj, clock->minute_handler);
+  if (clock->second_adj != NULL)
+    g_signal_handler_disconnect (clock->second_adj, clock->second_handler);
 
   gpe_clock_face_unprepare_xrender (widget);
 
@@ -608,8 +660,10 @@ gpe_clock_face_init (GpeClockFace *clock)
 
   clock->radius = DEFAULT_RADIUS;
 
+#ifdef HAVE_XRENDER
   clock->draw = NULL;
   clock->image_pict = clock->src_pict = 0;
+#endif
   clock->backing_pixmap = NULL;
   clock->backing_gc = NULL;
   clock->grabbed = FALSE;
@@ -672,12 +726,13 @@ gpe_clock_face_get_type (void)
 }
 
 GtkWidget *
-gpe_clock_face_new (GtkAdjustment *hour, GtkAdjustment *minute)
+gpe_clock_face_new (GtkAdjustment *hour, GtkAdjustment *minute, GtkAdjustment *second)
 {
   GpeClockFace *clock = g_object_new (gpe_clock_face_get_type (), NULL);
 
   clock->hour_adj = hour;
   clock->minute_adj = minute;
+  clock->second_adj = second;
 
   hand_angles (clock);
 
